function voiceKAML(inputfile, voiceoutfile,musicoutfile, nPeriods, harmonicStability,K)
    %*************************************************************************
    % Kernel Additive Modelling with light models
    %*************************************************************************
    % MATLAB implementation of the vocal/music extraction method presented in
    % "Scalable audio source separation with light KAM", submitted to ICASSP
    % ------------------------------------------------------------------------
    %
    % Separation is performed through spatial kernel backfitting of an
    % arbitrary number of sources, each one of them being described by its
    % own proximity kernel.
    %
    %
    %  parameters:
    %   * inputfile : path of the wav file to separate
    %   * voiceoutfile, musicoutfile : path of the output files to create
    %   * nPeriods : number of periodic components (5 by default)
    %   * harmonicStability: vector of dimension 2. Length in seconds of 
    %                        the harmonic kernel for stable parts in music,
    %                        for low and high frequencies. 
    %                        [1.3, 0.8] by default
    %   * K : number of components for compression. Default is 150
    %*************************************************************************
    % Copyright (C) 2014, Inria, Antoine Liutkus
    %
    %    This program is free software: you can redistribute it and/or modify
    %    it under the terms of the GNU Affero General Public License as
    %    published by the Free Software Foundation, either version 3 of the
    %    License, or (at your option) any later version.
    %
    %    This program is distributed in the hope that it will be useful,
    %    but WITHOUT ANY WARRANTY; without even the implied warranty of
    %    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    %    GNU Affero General Public License for more details.
    %
    %    You should have received a copy of the GNU Affero General Public License
    %    along with this program.  If not, see <http://www.gnu.org/licenses/>.


    %**************************************************************************
    %Parameters
    %**************************************************************************


    Lmax = inf; %length to consider in seconds
    posStart = 0; %or, start and stop positions if not nan.
    posStop = inf;


    %Signal and STFT
    windowLength = 80; % window length in ms
    overlap = 0.80;    % overlap ratio between 0 and 1
    alpha=1;  %target exponent for compression

    if nargin <6
        K = 20; %number of components for compact storage of models
    end
    regularization = 1E-5;

    %Parameters for the kernels
    %--------------------------
    %1) Repeating patterns (music)
    fMaxTempo = 18000; %max frequency to use for period selection
    if nargin <4
        nPeriods = 5;      %number of periods to separate
    end
    minPeriod = 0.7;     %in seconds
    maxPeriod = 30;    %in seconds

    %definition of the different frequency bands. Each one may have a different
    %stability parameter
    fstop_bands = [3000  ,22050];
    %The stability P indicates for how many periods each pattern is stable. A
    %high value will lead to the use of more points during the median fitlering
    %and thus will eliminate voice more. However, it will lead to a separated
    %voice which contains more music.
    P           = [3,     2]; %stability for each band

    %2) Stable harmonic source
    %stability in seconds for each band. If it is two small, it will catch the
    %voice. If it is too large, it will be useless and the voice will catch the
    %slowly evolving harmonic parts
    includeStableSourceModel = 1;
    if nargin <5
        harmonicStability = [1.5, 1];
    end

    %3) Vocals (lead)
    includeVoiceModel = 1;
    voiceKernel = [50, 0.03];% [Hertz, seconds], size of the voice kernel
    fMinVoice=60;      %min frequency of voice

    %Parameters for the backfitting algorithm
    %----------------------------------------
    niter= 4;          %number of iterations
    parallel = 0;       %parallel version. may be faster, but uses a lot of RAM

    %Display parameters
    %------------------
    displaySpectrograms = 0;
    displayPeriods = 0;

    %**************************************************************************
    % END OF PARAMETERS.
    %**************************************************************************

    %Closing parallel workers in case parallelization is not wanted
    if ~parallel&&exist('matlabpool')&&matlabpool('size')
        matlabpool close
    end

    %Loop over the files to separate

    [pathstr, name, ext] = fileparts(inputfile);
    fprintf('KAML: handling file %s...',inputfile);

    %Loading data
    %------------
    sig = Signal(inputfile);

    %Truncating audio if needs be
    %----------------------------
    len = size(sig.s,1);
    if ~isnan(posStart) && ~isnan(posStop)
        range = max(1,min(posStart*sig.fs,len)):max(1,min(posStop*sig.fs,len));
    else
        range = max(1,round(len/2-Lmax*sig.fs/2)):min(len, round(len/2+Lmax*sig.fs/2));
    end
    sig.s = sig.s(range,:);

    %Compute STFT
    %------------
    sig.windowLength = windowLength;
    sig.overlapRatio = overlap;
    sig.STFT;
    hopSize = sig.windowLength*(1-sig.overlapRatio)/1000;
    fprintf(' loaded, length: %0.2fs\n',sig.sLength/sig.fs);

    %Getting power spectrogram for periods estimation
    %------------------------------------------------
    X = single(sig.S(1:sig.nfftUtil,:,:));sig.S=[];
    V = single(max(eps,abs(X).^alpha)); %to the alpha th power (alpha harmonizable model)
    [F,T,I] = size(V);

    %Normalizing spectr ograms so that all frequencies have same mean energy
    %this is for the automatic period detection
    %----------------------------------------------------------------------
    weights = mean(V,2);
    V = single(bsxfun(@times,V,1./weights));

    %compute min and max periods in terms of number of frames
    %--------------------------------------------------------
    nminPeriod = ceil(minPeriod/hopSize);
    nmaxPeriod= min(floor(T/max(P)),ceil(maxPeriod/hopSize));
    Imax = floor(min(fMaxTempo,sig.fs)/sig.fs*F);
    if ~isempty(fstop_bands)
        poscuts=floor(min(fstop_bands,sig.fs)/sig.fs*sig.nfft);
    else
        poscuts = floor(sig.nfft/2);
    end

    %compute beat data, simple sum of autocorrelations of the lines of the
    %spectrogram: xcorr(x)=ifft(abs(fft(x)))
    %---------------------------------------------------------------------
    tempo = median(ifft(abs(fft((sum(V(1:Imax,:,:),3)'))).^2),2);

    tempo(nminPeriod:nmaxPeriod) = ...
        tempo(nminPeriod:nmaxPeriod).* linspace(1,0.5,nmaxPeriod-nminPeriod+1)';

    %Pick periods using magic peak peaking algorithm =)
    %--------------------------------------------------
    periods = pickpeaks(tempo(nminPeriod:nmaxPeriod),nPeriods,0)+nminPeriod-2;
    if displayPeriods
        figure(1);
        clf
        timePeriods= (0:length(tempo)-1)*hopSize;
        plot(timePeriods(nminPeriod:nmaxPeriod),tempo(nminPeriod:nmaxPeriod),'LineWidth',2);
        hold on;
        plot(timePeriods(periods+1),tempo(periods+1),'ro','LineWidth',2,'MarkerSize',15);
        grid on
        xlabel('period (s)');
        ylabel('spectral autocorrelation');
        legend('average auto-correlation ov','selected periods')
        title('Periods selection');
        drawnow;
    end

    %Creating kernels
    %----------------
    %1) for repeating part
    kernels = {};
    for j = 1:length(periods)
        %For all repeating patterns, kernel is a periodic line
        kernels{end+1} = {};
        for band = 1:length(poscuts)
            kernels{end}{band} = periodicKernel(periods(j),P(band),0,0);
        end
    end

    if includeStableSourceModel
        %2) Harmonic part
        kernels{end+1} = {};
        for band = 1:length(poscuts)
            nharmT = max(1,round(harmonicStability(band)/hopSize));
            kernels{end}{band}=ones(1,nharmT);
        end
    end

    if includeVoiceModel
        %3) For voice part, it's a 'cross' like
        %  0 1 0
        %  1 1 1
        %  0 1 0
        nvoiceF = max(1,round(voiceKernel(1)*sig.nfft/sig.fs));
        nvoiceT = max(1,round(voiceKernel(2)/hopSize));
        kernels{end+1} = zeros(2*nvoiceF+1,2*nvoiceT+1);
        kernels{end}(nvoiceF+1,:)=1;
        kernels{end}(:,nvoiceT+1)=1;
        %Computing min voice frequency
        IminVoice = floor(min(fMinVoice,sig.fs)/sig.fs*sig.nfft);
    end

    %J is the number of sources.
    J = length(kernels);

    tic

    % Initializing model
    %-------------------
    %PSD are initialized simply as mixtures PSD /J
    [W{1},~,H{1}] = randomSVD(sum(V,3)/J,K,'compact');
    clear V
    for j=2:J
        W{j} = W{1};
        H{j} = H{1};
    end

    %All spatial covariance matrices as initially identity
    R = zeros(1,I,I,J);
    for j = 1:J
        R(1,:,:,j) = eye(I);
    end
    R = repmat(R,F,[1,1,1]);

    %Performing estimation through kernel backfitting
    %--------------------------------------------------------
    str = [];
    for it=1:niter+1

        if it==niter+1            
            str = [];
            extractedSig = zeros(size(sig.s));
        end

        %Computing mixture covariance matrices
        Cxx = zeros(F,T,I,I,'single');
        for j=1:J
            Cxx=Cxx+ bsxfun(@times, permute(R(:,:,:,j),[1 4 2 3]),(W{j}*H{j}').^(2/alpha));
        end
        for i=1:I
            Cxx(:,:,i,i) = Cxx(:,:,i,i)+regularization;
        end

        %Computing inverses of mixture covariance matrices
        invCxx = zeros(F,T,I,I,'single');
        if I==1
            invCxx = 1./Cxx;
        elseif I==2
            invDet = 1./(Cxx(:,:,1,1).*Cxx(:,:,2,2) - Cxx(:,:,1,2).*Cxx(:,:,2,1));
            invCxx(:,:,1,1) = invDet.*Cxx(:,:,2,2);
            invCxx(:,:,2,1) = -invDet.*Cxx(:,:,2,1);
            invCxx(:,:,1,2) = -invDet.*Cxx(:,:,1,2);
            invCxx(:,:,2,2) = invDet.*Cxx(:,:,1,1);
        else
            error('This script only supports mono or stereo mixtures.');
        end
        clear Cxx;

        %Separating and backfitting each source
        for j=1:J
            if it==niter+1
                fprintf(repmat('\b',1,length(str)));
                str=sprintf('Rendering source %d / %d \n',j,J);
                fprintf('%s', str);
            else
                fprintf(repmat('\b',1,length(str)));
                str=sprintf('Backfitting, iteration %d / %d, source %d / %d \n',it,niter,j,J);
                fprintf('%s', str);
            end

            %Computing separated source j
            Ymuj = zeros(F,T,I,'single');
            Rj = R(:,:,:,j);

            G = zeros(F,T,I,I,'single');

            for i1=1:I
                for i2=1:I
                    Cyx_i1 = single(squeeze(bsxfun(@times,permute(Rj(:,i1,:),[1 5 2 3 4]),(W{j}*H{j}').^(2/alpha))));
                    G(:,:,i1,i2) = sum( bsxfun(@times, Cyx_i1, squeeze(invCxx(:,:,:,i2))),3);
                end
            end

            for i1 = 1:I
                Ymuj(:,:,i1) = sum( bsxfun(@times,squeeze( G(:,:,i1,:)), X),3);
            end

            if it==niter+1
                %Rendering each each source, and adding it to the background if a
                %mixdown is asked for, or exporting its wave in the contrary case
                sig.S = sig.buildComplete(Ymuj);
                extractedSig = extractedSig + sig.iSTFT();
                if j<J, wavefile = musicoutfile; else wavefile = voiceoutfile; end
                if (j>=J-1)
                    wavwrite(extractedSig,sig.fs,wavefile);
                    extractedSig = zeros(size(sig.s));
                end
                continue
            end

            %Now backfitting model

            %learning spatial covariance and PSD from separated image
            [Z,Rj] = learnDSPlight(Ymuj);

            %nan or inf will occur if all PSD estimates are 0, typical for voice below
            %minimal frequency
            Rj(isnan(Rj)) = 0;
            Rj(isinf(Rj)) = 0;
            R(:,:,:,j)=Rj;

            %median filter the estimated PSD Z
            if iscell(kernels{j})
                %we handle the case were we have several frequency bands
                pos = 1;
                for k = 1:length(poscuts)
                    order= round(length(find(kernels{j}{k}))/2);
                    Z(pos:poscuts(k),:) = ordfilt2(Z(pos:poscuts(k),:),order,kernels{j}{k},'symmetric');
                    pos = poscuts(k)+1;
                end
            else
                order= round(length(find(kernels{j}))/2);
                Z = ordfilt2(Z,order,kernels{j},'symmetric');
            end


            if includeVoiceModel && fMinVoice && (j==J)
                %if source is voice, then set to 0 its low frequencies if
                %needed
                Z(1:IminVoice,:) = 0;
            end
            [W{j},~,H{j}] = randomSVD(Z.^(alpha/2),K,'compact');

        end

        %display if needed
        if displaySpectrograms
            figure(2)
            clf
            for j=1:J
                prows =  min(factor(J));
                if prows == J
                    subplot(1,J,j);
                else
                    subplot(prows,J/prows,j);
                end
                %imagesc(log(max(1E-10,S(:,:,j))))
                imagesc(real(W{j}*H{j}'))
                colorbar
            end
            drawnow
            pause(0.02)
        end
    end

    timeTaken = toc;
    fileLength = sig.sLength/sig.fs;
    fprintf('KAML: done. %0.1fs : computing speed %0.2f s/s \n',timeTaken,fileLength/timeTaken)
end


function [P,R] = learnDSPlight(S)
    %--------------------------------------------------------------------------
    % Learn Power Spectral Density and spatial covariance matrix of a
    % multichannel image
    %
    % Input :
    %   * S : FxTxJ : STFT or MDCT of the multichannel image
    %
    % Output :
    %   * P : FxT   : estimated DSP of signal
    %   * R : FxJxJ : estimated spatial covariance matrix
    %--------------------------------------------------------------------------
    % Copyright (C) 2014, Inria, Antoine Liutkus
    %
    %    This program is free software: you can redistribute it and/or modify
    %    it under the terms of the GNU Affero General Public License as
    %    published by the Free Software Foundation, either version 3 of the
    %    License, or (at your option) any later version.
    %
    %    This program is distributed in the hope that it will be useful,
    %    but WITHOUT ANY WARRANTY; without even the implied warranty of
    %    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    %    GNU Affero General Public License for more details.
    %
    %    You should have received a copy of the GNU Affero General Public License
    %    along with this program.  If not, see <http://www.gnu.org/licenses/>.


    [F,T,J] = size(S);

    P = max(eps,mean(abs(S).^2,3));


    R = zeros(1,J,J);
    R(1,:,:) = eye(J);
    R = repmat(R,F,[1,1,1]);

    EYE = R*1E-5;

    for j1 = 1:J
        for j2 = 1:J
            R(:,j1,j2) = mean(bsxfun(@times, 1./max(eps,P),S(:,:,j1).*conj(S(:,:,j2))),2)+EYE(:,j1,j2);
        end
    end

    Rinv = zeros(size(R));
    for f = 1:F
        R(isnan(R))=0;
        R(isinf(R))=0;
        Rinv(f,:,:) = pinv(squeeze(R(f,:,:)));
    end

    P = zeros(F,T);
    for j=1:J
        for j2 = 1:J
            P = P + real(bsxfun(@times, Rinv(:,j,j2), S(:,:,j2).*conj(S(:,:,j))));
        end
    end
    P = P/J;
end


function [peaks,criterion] = pickpeaks(V,select,display)
    % -------------------------------------------------------------
    % Scale-space peak picking
    % ------------------------
    % This function looks for peaks in the data using scale-space theory. 
    % 
    % input : 
    %   * V : data, a vector
    %   * select : either: 
    %       - select >1 : the number of peaks to detect
    %       - 0<select<1 : the threshold to apply for finding peaks 
    %         the closer to 1, the less peaks, the closer to 0, the more peaks
    %   * display : whether or not to display a figure for the results. 0 by
    %               default
    %   * ... and that's all ! that's the cool thing about the algorithm =)
    % 
    % outputs :
    %   * peaks : indices of the peaks
    %   * criterion : the value of the computed criterion. Same
    %                 length as V and giving for each point a high value if
    %                 this point is likely to be a peak
    % 
    % The algorithm goes as follows:
    % 1°) set a smoothing horizon, initially 1;
    % 2°) smooth the data using this horizon
    % 3°) find local extrema of this smoothed data
    % 4°) for each of these local extrema, link it to a local extremum found in
    %     the last iteration. (initially just keep them all) and increment the 
    %     corresponding criterion using current scale. The
    %     rationale is that a trajectory surviving such smoothing is an important
    %     peak
    % 5°) Iterate to step 2°) using a larger horizon.
    % 
    % At the end, we keep the points with the largest criterion as peaks.
    % I don't know if that kind of algorithm has already been published
    % somewhere, I coded it myself and it works pretty nice, so.. enjoy !
    % If you find it useful, please mention it in your studies =)
    % 
    % running time should be decent, although intrinsically higher than 
    % findpeaks. For vectors of length up to, say, 10 000, it should be nice. 
    % Above, it may be worth it though.
    % ---------------------------------------------------------------------
    % Copyright (C) 2014, Inria, Antoine Liutkus
    %
    %Redistribution and use in source and binary forms, with or without
    %modification, are permitted provided that the following conditions are met:
    %    * Redistributions of source code must retain the above copyright
    %      notice, this list of conditions and the following disclaimer.
    %    * Redistributions in binary form must reproduce the above copyright
    %      notice, this list of conditions and the following disclaimer in the
    %       documentation and/or other materials provided with the distribution.
    %     * Neither the name of Inria nor the names of its contributors may 
    %       be used to endorse or promote products derived from this software 
    %       without specific prior written permission.
    % 
    % THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
    % ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    % WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    % DISCLAIMED. IN NO EVENT SHALL INRIA BE LIABLE FOR ANY
    % DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    % (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    % LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    % ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    % (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    % SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    %data is a vector
    V = V(:)-min((V(:)));

    %input parsin
    if nargin < 3
        display=0;
    end
    if nargin < 2
        select= 0;
    end

    n = length(V);

    %definition of local variables
    buffer = zeros(n,1);
    criterion = zeros(n,1);
    if select < 1
        minDist = n/20;
    else
        minDist = n/select;
    end
    %horizons = round(linspace(1,ceil(n/20),50));
    horizons = unique(round(logspace(0,2,50)/100*ceil(n/20)));

    %horizons=1:2:50;
    Vorig = V;

    % all this tempMat stuff is to avoid calling findpeaks which is horribly
    % slow for our purpose
    tempMat = zeros(n,3);
    tempMat(1,1)=inf;
    tempMat(end,3)=inf;

    % loop over scales
    for is=1:length(horizons)

        %sooth data, using fft-based convolution with a half sinusoid
        horizon = horizons(is);
        if horizon > 1
            w=max(eps,sin(2*pi*(0:(horizon-1))/2/(horizon-1)));
            w=w/sum(w);    
            %V=conv(V(:),w(:),'same');
            V = real(ifft(fft(V(:),n+horizon).*fft(w(:),n+horizon)));
            V = V(1+floor(horizon/2):end-ceil(horizon/2));
        end

        %find local maxima
        tempMat(2:end,1) = V(1:end-1);
        tempMat(:,2) = V(:);
        tempMat(1:end-1,3) = V(2:end);
        [~,posMax] =max(tempMat,[],2);
        I = find(posMax==2);
        I = I(:)';

        %initialize buffer
        newBuffer = zeros(size(buffer));

        if is == 1
            % if first iteration, keep all local maxima
            newBuffer(I) = Vorig(I);
        else    
            old = find(buffer);
            old = old(:)';
            if isempty(old)
                continue;
            end

            %Now, for each element of I, find the closest element in
            %old along with its distance. The few nice lines below were
            %written by Roger Stafford in a forum post available here:
            %http://www.mathworks.fr/matlabcentral/newsreader/view_thread/24387
            [c,p] = sort(old);
            [~,ic] = histc(I,[-inf,(c(1:end-1)+c(2:end))/2,inf]);
            iOld = p(ic);
            d = abs(I-old(iOld));

            %done, now select only those that are sufficiently close
            neighbours = iOld(d<minDist);

            if ~isempty(neighbours)
                newBuffer(old(neighbours)) = V(old(neighbours))*is^2;
            end
        end
        %update stuff
        buffer = newBuffer;
        criterion = criterion + newBuffer;
    end

    %normalize criterion
    criterion = criterion/max(criterion);

    %find peaks based on criterion
    if select<1
        peaks = find(criterion>select);
    else
    %     sorted = find(criterion>1E-3);
    %     [~,order] = sort(criterion(sorted),'descend');
    %     peaks = sorted(order(1:min(length(sorted),select)));
        [~,order] = sort(criterion,'descend');
        peaks = order(1:select);
    end

    if display
        %display
        clf
        plot(Vorig,'LineWidth',2);
        hold on
        plot(criterion*max(Vorig),'r');
        hold on
        plot(peaks,Vorig(peaks),'ro','MarkerSize',10,'LineWidth',2)
        grid on
        title('Scale-space peak detection','FontSize',16);
        legend('data','computed criterion','selected peaks');
    end
end

function K = periodicKernel(period,P, V,H)
    %Builds a periodic kernel, looking like this:
    %10001000100010001

    temp = strel('periodicline',P,[0, period]);
    L = 2*P*period+1;
    K = zeros(2*V+1,L);
    K(V+1,:)=temp.getnhood;
    K(:,P*period+1-H:P*period+1+H)=1;
end

function [U,S,V] = randomSVD(A,K,mode)
    %magic truncated SVD algorithm (see Halko 'finding structure with
    %randomness etc')
    %
    % mode is either:
    % - 'normal' (default) : S is a KxK diagonal matrix
    % - 'diagonal : S is the Kx1 vector gathering the singular values
    % - 'compact" : U and V are both multiplied by sqrt(S), than S is set to 1.

    if nargin<3
        mode = 'normal';
    end

    [M,N] = size(A);
    P = min(2*K,N);
    X = randn(N,P);
    Y = A*X;
    W1 = orth(Y);
    B = W1'*A;
    [W2,S,V] = svd(B,'econ');
    U = W1*W2;
    K=min(K,size(U,2));
    U = U(:,1:K);
    S = S(1:K,1:K);
    V=V(:,1:K);

    if strcmp(mode,'diagonal')
        S=diag(S);
    elseif strcmp(mode,'compact')
        sqrtS = diag(sqrt(diag(S)));
        U=U*sqrtS;
        V=V*sqrtS;
        S = eye(K);
    end
end